@@ -7,16 +7,12 @@ files and command-line parameters.
 As of March 22, 2022, configpile is still pretty much influenced by :mod:`argparse`, and the
 machinery below supports a subset of :mod:`argparse` functionality. Later on, we may cut ties
 with :mod:`argparse`, add our own help/usage message writing, our own Sphinx extension and
-encourage extending those processing classes. 
+encourage extending those processing classes.
 
 .. rubric:: Types
 
 This module uses the following types.
 
-.. py:data:: _Value
-
-    Value being parsed by a :class:`.ParamType`
-
 .. py:data:: _Config
 
     Type of the configuration dataclass to construct
@@ -28,10 +24,8 @@ import argparse
 import configparser
 import sys
 import warnings
-from abc import ABC, abstractmethod
 from configparser import ConfigParser
 from dataclasses import dataclass
-from enum import Enum
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
@@ -45,7 +39,6 @@ from typing import (
     Mapping,
     Optional,
     Sequence,
-    Tuple,
     Type,
     TypeVar,
     Union,
@@ -53,8 +46,10 @@ from typing import (
 
 from typing_extensions import Annotated, get_args, get_origin, get_type_hints
 
-from .arg import Arg, Expander, Param, Positional
-from .userr import Err, Res, in_context
+from .arg import Arg, Expander, Param
+from .enums import SpecialAction
+from .handlers import CLHandler, CLPos, CLSpecialAction, CLStdHandler, KVHandler
+from .userr import Err, Res
 from .util import ClassDoc, filter_types_single
 
 if TYPE_CHECKING:
@@ -62,266 +57,6 @@ if TYPE_CHECKING:
 
 _Config = TypeVar("_Config", bound="Config")
 
-_Value = TypeVar("_Value")
-
-
-class CLHandler(ABC):
-    """
-    A handler for command-line arguments
-    """
-
-    @abstractmethod
-    def handle(self, args: Sequence[str], state: State) -> Tuple[Sequence[str], Optional[Err]]:
-        """
-        Processes arguments, possibly updating the state or returning errors
-
-        Args:
-            args: Command-line arguments not processed yet
-            state: (Mutable) state to possibly update
-
-        Returns:
-            The updated command-line and an optional error
-        """
-        pass
-
-
-@dataclass(frozen=True)
-class CLSpecialAction(CLHandler):
-    """
-    A handler that sets the special action
-    """
-
-    special_action: SpecialAction  #: Special action to set
-
-    def handle(self, args: Sequence[str], state: State) -> Tuple[Sequence[str], Optional[Err]]:
-        if state.special_action is not None:
-            before = state.special_action.name
-            now = self.special_action.name
-            err = Err.make(f"We had already action {before}, conflicts with action {now}")
-            return (args, err)
-        state.special_action = self.special_action
-        return (args, None)
-
-
-@dataclass(frozen=True)
-class CLInserter(CLHandler):
-    """
-    Handler that expands a flag into a sequence of args inserted into the command line to be parsed
-    """
-
-    #: Arguments inserted in the command-line
-    inserted_args: Sequence[str]
-
-    def handle(self, args: Sequence[str], state: State) -> Tuple[Sequence[str], Optional[Err]]:
-        # TODO: run the command line parser on the inserted args
-        return ([*self.inserted_args, *args], None)
-
-
-@dataclass(frozen=True)
-class CLParam(CLHandler, Generic[_Value]):
-    """
-    Parameter handler
-
-    Takes a single string argument from the command line, parses it and pushes into the
-    corresponding sequence of instances
-    """
-
-    #: Parameter to handle
-    param: Param[_Value]
-
-    def action(self, value: _Value, state: State) -> Optional[Err]:
-        """
-        A method called on the successful parse of a value
-
-        Can be overridden. By default does nothing.
-
-        Args:
-            value: Parsed value
-            state: State to possibly update
-
-        Returns:
-            An optional error
-        """
-        return None
-
-    def handle(self, args: Sequence[str], state: State) -> Tuple[Sequence[str], Optional[Err]]:
-        if args:
-            res = self.param.parser.parse(args[0])
-            if isinstance(res, Err):
-                return (args[1:], res.in_context(param=self.param.name))
-            else:
-                assert self.param.name is not None, "Names are assigned after initialization"
-                err = in_context(self.action(res, state), param=self.param.name)
-                state.instances[self.param.name] = [*state.instances[self.param.name], res]
-                return (args[1:], err)
-        else:
-            return (
-                args,
-                Err.make("Expected value, but no argument present", param=self.param.name),
-            )
-
-
-@dataclass(frozen=True)
-class CLConfigParam(CLParam[Sequence[Path]]):
-    """
-    A configuration file parameter handler
-
-    If paths are successfully parsed, it appends configuration files to be parsed to the current
-    state.
-    """
-
-    def action(self, value: Sequence[Path], state: State) -> Optional[Err]:
-        state.config_files_to_process.extend(value)
-        return None
-
-
-@dataclass
-class CLPos(CLHandler):
-    """
-    Handles positional parameters
-
-    Note that this handler has state, namely the positional parameters that are still expected.
-    """
-
-    pos: List[Param[Any]]  #: (Mutable) list of positional parameters
-
-    @staticmethod
-    def make(seq: Sequence[Param[Any]]) -> CLPos:
-        """
-        Constructs a positional parameter handler from a sequence of positional parameters
-
-        Args:
-            seq: Positional parameters
-
-        Returns:
-            Handler
-        """
-        assert all([p.positional is not None for p in seq]), "All parameters should be positional"
-        assert all(
-            [not p.positional.should_be_last() for p in seq[:-1] if p.positional is not None]
-        ), "Positional parameters with a variable number of arguments should be last"
-        l = list(seq)  # makes a mutable copy
-        return CLPos(l)
-
-    def handle(self, args: Sequence[str], state: State) -> Tuple[Sequence[str], Optional[Err]]:
-        if not args:
-            return (args, None)  # should not happen ,but let's not crash
-        if not self.pos:
-            return (args[1:], Err.make(f"Unknown argument {args[0]}"))
-        p = self.pos[0]
-        assert p.name is not None
-        res = p.parser.parse(args[0])
-        if isinstance(res, Err):
-            return (args[1:], in_context(res, param=p.name))
-        else:
-            state.append(p.name, res)
-            if p.positional == Positional.ONCE:
-                self.pos = self.pos[1:]
-            return (args[1:], None)
-
-
-@dataclass(frozen=True)
-class CLStdHandler(CLHandler):
-    """
-    The standard command line arguments handler
-
-    It processes arguments one by one. If it recognizes a flag, the corresponding handler is
-    called. Otherwise, control is passed to the fallback handler, which by default processes
-    positional parameters.
-    """
-
-    flags: Mapping[str, CLHandler]
-    fallback: CLHandler
-
-    def handle(self, args: Sequence[str], state: State) -> Tuple[Sequence[str], Optional[Err]]:
-        if not args:
-            return (args, None)
-        flag = args[0]
-        handler = self.flags.get(flag)
-        if handler is not None:
-            next_args, err = handler.handle(args[1:], state)
-            err = in_context(err, flag=flag)
-            return next_args, err
-        else:
-            return self.fallback.handle(args, state)
-
-
-class KVHandler(ABC):
-    """
-    Handler for key/value pairs found for example in environment variables or INI files
-
-    Note that the key is not stored/processed in this class.
-    """
-
-    @abstractmethod
-    def handle(self, value: str, state: State) -> Optional[Err]:
-        """
-        Processes
-
-        Args:
-            value: Value to parse and process
-            state: State to update
-
-        Returns:
-            An error if an error occurred
-        """
-        pass
-
-
-@dataclass(frozen=True)
-class KVParam(KVHandler, Generic[_Value]):
-    """
-    Handler for the value following a key corresponding to a parameter
-    """
-
-    #: Parameter to handle
-    param: Param[_Value]
-
-    def action(self, value: _Value, state: State) -> Optional[Err]:
-        """
-        A method called on the successful parse of a value
-
-        Can be overridden. By default does nothing.
-
-        Args:
-            value: Parsed value
-            state: State to possibly update
-
-        Returns:
-            An optional error
-        """
-        return None
-
-    def handle(self, value: str, state: State) -> Optional[Err]:
-        res = self.param.parser.parse(value)
-        if isinstance(res, Err):
-            return res
-        else:
-            assert self.param.name is not None
-            err = self.action(res, state)
-            state.instances[self.param.name] = [*state.instances[self.param.name], res]
-            return in_context(err, param=self.param.name)
-
-
-@dataclass(frozen=True)
-class KVConfigParam(KVParam[Sequence[Path]]):
-    """
-    Handler for the configuration file value following a key corresponding to a parameter
-    """
-
-    def action(self, value: Sequence[Path], state: State) -> Optional[Err]:
-        state.config_files_to_process.extend(value)
-        return None
-
-
-class SpecialAction(Enum):
-    """
-    Describes special actions that do not correspond to normal execution
-    """
-
-    HELP = "help"  #: Display a help message
-    VERSION = "version"  #: Print the version number
-
 
 @dataclass
 class State:
@@ -409,10 +144,10 @@ class IniProcessor:
                                 err = Err.make(f"Unknown key {key}")
                         if err is not None:
                             errors.append(err.in_context(ini_section=section_name))
-        except configparser.Error as e:
-            errors.append(Err.make(f"Parse error"))
-        except IOError as e:
-            errors.append(Err.make(f"IO Error"))
+        except configparser.Error:
+            errors.append(Err.make("Parse error"))
+        except IOError:
+            errors.append(Err.make("IO Error"))
         return errors
 
     def process_string(self, ini_contents: str, state: State) -> Optional[Err]:
@@ -431,10 +166,10 @@ class IniProcessor:
         try:
             parser.read_string(ini_contents)
             errors.extend(self._process(parser, state))
-        except configparser.Error as e:
-            errors.append(Err.make(f"Parse error"))
-        except IOError as e:
-            errors.append(Err.make(f"IO Error"))
+        except configparser.Error:
+            errors.append(Err.make("Parse error"))
+        except IOError:
+            errors.append(Err.make("IO Error"))
         if errors:
             return Err.collect(*errors)
         else:
@@ -458,13 +193,13 @@ class IniProcessor:
             return Err.make(f"Path {ini_file_path} is not a file")
         parser = ConfigParser()
         try:
-            with open(ini_file_path, "r") as file:
+            with open(ini_file_path, "r", encoding="utf-8") as file:
                 parser.read_file(file)
                 errors.extend(self._process(parser, state))
-        except configparser.Error as e:
-            errors.append(Err.make(f"Parse error"))
-        except IOError as e:
-            errors.append(Err.make(f"IO Error"))
+        except configparser.Error:
+            errors.append(Err.make("Parse error"))
+        except IOError:
+            errors.append(Err.make("IO Error"))
         if errors:
             return Err.collect(*errors)
         else:
@@ -538,7 +273,7 @@ class ProcessorFactory(Generic[_Config]):
             formatter_class=argparse.RawTextHelpFormatter,
             add_help=False,
         )
-        argument_parser._action_groups.pop()
+        argument_parser._action_groups.pop()  # pylint: disable=protected-access
         return ProcessorFactory(
             params_by_name={},
             argument_parser=argument_parser,
@@ -608,10 +343,10 @@ class Processor(Generic[_Config]):
             if arg is not None:
                 help_lines = docs[name]
                 if help_lines is None:
-                    help = ""
+                    hlp = ""
                 else:
-                    help = "\n".join(help_lines)
-                arg = arg.updated(name, help, config_type.env_prefix_)
+                    hlp = "\n".join(help_lines)
+                arg = arg.updated(name, hlp, config_type.env_prefix_)
                 args.append(arg)
         return args
 
